function output = rddik_09(X,Y,c)

%--------------------------------------------------------------------------
% rddik_09.m: compute the sharp regression discontinuity estimator by IK
%               Procedures based on the working paper versin in 2009
%--------------------------------------------------------------------------
%
% DESCRIPTION:
%   Estimate the sharp regression discontinuity estimator with the
%   bandwidths proposed by IK.
%
% 
% INPUT ARGUMENTS
%   X: Imput vector (n by 1) of the forcing variable 
%   Y: Vector of the outcome variable (n by 1) 
%   c: Evaluation (discontinuity) point (scalar)
%
% OUTPUT ARGUMENTS:
%
% 
%
% 
% AUTHOR:
%   This code was written by Yoichi Arai.
%   I will appreciate any feedbacks.
%
% Functions required for rddmmse.m
%
% 
%
% 
% DEVELOPMENT: 
%    18 Nov 2013: Original version of rddmmse.m written.
%
%--------------------------------------------------------------------------

% Check input arguments
if nargin < 3
    error('more input arguments needed.');
end

if nargin > 3
    error('Too many arguments.');
end

    %%%%%%%%%%%%%%%%%%%%%%%%
    % Step 1
    %%%%%%%%%%%%%%%%%%%%%%%%
    n = size(X,1);
    Sx = sqrt(var(X));
    h1 = 1.84*Sx*n^(-1/5);      % h1
    
    Xbin_bt_xh1 = (X>=c-h1);     % one for observation bigger than x-h1, 0 otherwise
    Xbin_lt_x = (X<c);           % N- in p.16
    Xbin_bt_x = (X>=c);          % N+
    Xbin_lt_xh1 = (X<=c+h1);

    num_X_bt_x = sum(Xbin_bt_x); % # of observations of X bigger than x
    num_X_lt_x = sum(Xbin_lt_x); % # of observations of X less than x
    if (num_X_bt_x==0)||(num_X_lt_x==0)
        error('# of observations either bigger or less than x is zero')
    end
    
    Xbin_m = Xbin_bt_xh1.*Xbin_lt_x;    % index for X in [x-h,x]
    Xbin_p = Xbin_bt_x.*Xbin_lt_xh1;    % index for X in [x,x+h]
    Nh1_m = sum(Xbin_m);                % # of X in [x-h,x]
    Nh1_p = sum(Xbin_p);                % # of X in [x,x+h]
    if (Nh1_m==0)||(Nh1_p==0)||(Nh1_m==1)||(Nh1_p==1)
        error('# of observations is too few for step 1')
    end
    Nh1_sum = Nh1_m + Nh1_p;            % # of X in [x-h,x+h]
    
    % a. density estimate for X (kernel density estimation)
    % Method 1. Naive esimate by IK
    f_c = Nh1_sum/(2*n*h1);              % f_c in (4.11)
    Yh1_bar_m = sum(Y.*Xbin_m)/Nh1_m;
    Yh1_bar_p = sum(Y.*Xbin_p)/Nh1_p;

    
    vy_m = sum(((Y-Yh1_bar_m).*Xbin_m).^2)/(Nh1_m-1);
    vy_p = sum(((Y-Yh1_bar_p).*Xbin_p).^2)/(Nh1_p-1);
    vc2 = (vy_m*(Nh1_m-1)+vy_p*(Nh1_p-1))/Nh1_sum;

    %%%%%%%%%%%%%%%%%%%%%%%%
    % Step 2
    %%%%%%%%%%%%%%%%%%%%%%%%

    % 3rd order global polynomial regression in page 13 of IK(09)
    Xx = X-c;
    Xx2 = Xx.^2;
    Xx3 = Xx.*Xx2;
    nones = ones(n,1);
    data_mat = [Y nones Xx Xx2 Xx3];
    X_s2_p = X(Xbin_bt_x==1,:);
    X_s2_m = X(Xbin_lt_x==1,:);
    median_p = median(X_s2_p);
    median_m = median(X_s2_m);   
    middle = (X>=median_m)&(X<=median_p);
    data_step2 = data_mat(middle,:);
    X_step2 = [data_step2(:,2) (data_step2(:,3)>=0) data_step2(:,3:5)];
    Y_step2 = data_step2(:,1);
    beta_step2 = X_step2\Y_step2;
    gamma_4 = 6*beta_step2(5,1);
    
    % 2nd order LPR with uniform kernel
    h2_p = 3.56*(vc2/f_c/max(gamma_4^2,0.01)/num_X_bt_x)^(1/7);
    h2_m = 3.56*(vc2/f_c/max(gamma_4^2,0.01)/num_X_lt_x)^(1/7);
    Xbin_bet_xxh = (X>=c).*(X<=c+h2_p);     % index for X in [x, x+h2_p]
    data_trim_mat2_p = data_mat(Xbin_bet_xxh==1,:);
    YT_p = data_trim_mat2_p(:,1);
    T_p = data_trim_mat2_p(:,2:4);
    lambda_p_all = T_p\YT_p;
    m2c_p = 2*lambda_p_all(3,1);

    Xbin_bet_xhx = (X>=c-h2_m).*(X<=c);     % index for X in [x-h2_m, x]
    data_trim_mat2_m = data_mat(Xbin_bet_xhx==1,:);
    YT_m = data_trim_mat2_m(:,1);
    T_m = data_trim_mat2_m(:,2:4);
    lambda_m_all = T_m\YT_m;
    m2c_m = 2*lambda_m_all(3,1);

    %%%%%%%%%%%%%%%%%%%%%%%%
    % Step 3
    %%%%%%%%%%%%%%%%%%%%%%%%
    num_X_bet_xxh = sum(Xbin_bet_xxh);
    num_X_bet_xhx = sum(Xbin_bet_xhx);

    r_p = 720*vc2/(num_X_bet_xxh*h2_p^4);
    r_m = 720*vc2/(num_X_bet_xhx*h2_m^4);

    h3 = 3.4375*(2*vc2/(n*f_c*((m2c_p - m2c_m)^2+r_p+r_m)))^(1/5);
        
    %%%%%%%%%%%%%%%%%%%%%%%%
    % Step 4 Get RD estimate
    %%%%%%%%%%%%%%%%%%%%%%%%
    [tau,se,delta_p,delta_m,Num] = srdd_se(Y,X,c,h3,h3);
    
    % Compute bias correction terms b11 and b12
    Xbin_bt_xh_opt = (X >= c-h3);
    Xbin_lt_xh_opt = (X <= c+h3);
    Xbin_bet_xxh_opt = Xbin_bt_x.*Xbin_lt_xh_opt;    % index for X in [x-h,x]
    Xbin_bet_xhx_opt = Xbin_bt_xh_opt.*Xbin_lt_x;    % index for X in [x,x+h]

    data_trim_mat_p_opt = data_mat(Xbin_bet_xxh_opt==1,:);
    T_p_opt = data_trim_mat_p_opt(:,2:3);
    Kvec_p_opt = (1-abs(data_trim_mat_p_opt(:,3))/h3).*(abs(data_trim_mat_p_opt(:,3))<h3);
    W_p_opt = diag(Kvec_p_opt);

    data_trim_mat_m_opt = data_mat(Xbin_bet_xhx_opt==1,:);
    T_m_opt = data_trim_mat_m_opt(:,2:3);
    Kvec_m_opt = (1-abs(data_trim_mat_m_opt(:,3))/h3).*(abs(data_trim_mat_m_opt(:,3))<h3);
    W_m_opt = diag(Kvec_m_opt);

    Sn01_p_tilde_inv_opt = invpd(T_p_opt'*W_p_opt*T_p_opt);
    cn2_p_tilde_opt = data_trim_mat_p_opt(:,4:5)'*Kvec_p_opt;
    b11_vec_opt = m2c_p*Sn01_p_tilde_inv_opt*cn2_p_tilde_opt/2;

    Sn01_m_tilde_inv_opt = invpd(T_m_opt'*W_m_opt*T_m_opt);
    cn2_m_tilde_opt = data_trim_mat_m_opt(:,4:5)'*Kvec_m_opt;
    b12_vec_opt = m2c_m*Sn01_m_tilde_inv_opt*cn2_m_tilde_opt/2;

    bias_opt = b11_vec_opt(1,1)-b12_vec_opt(1,1);
    
    output.tau = tau;
    output.se = se;
    output.h = h3;
    output.tval =(tau-bias_opt)/se;
    output.ci = [tau-bias_opt-1.96*se tau-bias_opt+1.96*se];
    output.delta = [delta_p delta_m];
    output.bias = bias_opt;
    output.n = Num;
    

% 
% function [tau,se,delta_p,delta_m,n_p,n_m] = srdd_est(Y,X,c,h_p,h_m)
% 
% N=size(Y,1);
% Xx = X - c;
% data_mat = [Y ones(N,1) Xx];
% 
% Xbin_bet_xh = (X>=c).*(X<=c+h_p);     % index for X in [x, x+h2_p]
% n_p = sum(Xbin_bet_xh);
% data_trim_mat_p = data_mat(Xbin_bet_xh==1,:);
% YT_p = data_trim_mat_p(:,1);
% T_p = data_trim_mat_p(:,2:3);
% Kvec_p = (1-abs(data_trim_mat_p(:,3))/h_p).*(abs(data_trim_mat_p(:,3))<h_p);
% W_p = diag(Kvec_p);
% lambda_p_all = (T_p'*W_p*T_p)\(T_p'*W_p*YT_p);
% delta_p = lambda_p_all(1,1);
% Resid_p = YT_p - T_p*lambda_p_all;
% %Sigma_p = diag(Resid_p.^2);
% Sigma_p_homo = sum(Resid_p.^2)/(size(Resid_p,1)-2);
% Gam_inv_p = invpd(T_p'*W_p*T_p); 
% V_mat_p = Sigma_p_homo*Gam_inv_p*(T_p'*W_p*W_p*T_p)*Gam_inv_p;
% 
% Xbin_bet_hx = (X<c).*(X>=c-h_m);     % index for X in [x, x+h2_p]
% n_m = sum(Xbin_bet_hx);
% data_trim_mat_m = data_mat(Xbin_bet_hx==1,:);
% YT_m = data_trim_mat_m(:,1);
% T_m = data_trim_mat_m(:,2:3);
% Kvec_m = (1-abs(data_trim_mat_m(:,3)/h_m)).*(abs(data_trim_mat_m(:,3))<h_m);
% W_m = diag(Kvec_m);
% lambda_m_all = (T_m'*W_m*T_m)\(T_m'*W_m*YT_m);
% delta_m = lambda_m_all(1,1);
% Resid_m = YT_m - T_m*lambda_m_all;
% %Sigma_m = diag(Resid_m.^2);
% Sigma_m_homo = sum(Resid_m.^2)/(size(Resid_m,1)-2);
% Gam_inv_m = invpd(T_m'*W_m*T_m); 
% V_mat_m = Sigma_m_homo*Gam_inv_m*(T_m'*W_m*W_m*T_m)*Gam_inv_m;
% variance = V_mat_p(1,1)+V_mat_m(1,1);
% 
% tau = delta_p - delta_m;
% se = sqrt(variance);


%------------------------------------------------------------------
% End private function srdd
%------------------------------------------------------------------