%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Simulation to generate the result of Table 1
%
% D: DGP design
% x: cut-off point
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear;

% Simulation design
M = 10000;      % # of iteration
n = 2000;        % sample size
x_ep = 0;       % cut-off point
D_num = 4;      % number of DGPs
methods = 4;    % number of bandwidth selection methods
num_ini = 10;   % number of initial values tried in optimization
kernel = 2;     % 1: Uniform % 2: Triangular

% Functions used in this code (in Auxiliary code folder)
%   CCT_B.m, Const_LPR_U.m, Const_LPR.m, data_RDD.m, density_est.m, density1d.m, nnm.m, Qn_AI_h_RDD.m, rddcct.m, rddik_09.m, srdd.m

% Get constants based on equivalent kernel for plug-in bandwidth
 C23K_U = Const_LPR_U(2,3);
 C33K_U = Const_LPR_U(3,3);
 C34K_U = Const_LPR_U(3,4);

 C23K_T = Const_LPR(2,3);
 C33K_T = Const_LPR(3,3);
 C34K_T = Const_LPR(3,4);

 k_const = [-0.1 -0.1 -0.08 24/5]; % [b1 c1 c2 v]
            
% Initialization
h_mat_1 = zeros(M,1);
h_mat_2 = zeros(M,2);
h_mat_3 = zeros(M,1);
h_mat_4 = zeros(M,2);

b_mat_cct = zeros(M,1);
h23_mat = zeros(M,2);
h3_mat = zeros(M,2);

tau_bias_mat_1 = zeros(M,1);
tau_bias_mat_2 = zeros(M,1);
tau_bias_mat_3 = zeros(M,1);
tau_bias_mat_4 = zeros(M,1);

delta_bias_mat_1 = zeros(M,2);
delta_bias_mat_2 = zeros(M,2);
delta_bias_mat_3 = zeros(M,2);
delta_bias_mat_4 = zeros(M,2);

tic

%
% Begin Monte Carlo
%

for D=1:4
rng(10);

   
        
for ite = 1:M
    % Data generation
    [Y,X,Tmat] = data_RDD(n,D);
    true_tau  = Tmat(1,5);
    true_delta_p = Tmat(1,10);
    true_delta_m = Tmat(1,11);
    delta_true = [true_delta_p true_delta_m];
    
    % Initialization for global maximization
    h_mat_local_4 = zeros(num_ini,2);
    min_val_local_4 = zeros(num_ini,1);

    
% 1. IK bandwidth (2009 Working paper version)
%
    %%%%%%%%%%%%%%%%%%%%%%%%
    % Step 1
    %%%%%%%%%%%%%%%%%%%%%%%%

    Sx = sqrt(var(X));
    h1 = 1.84*Sx*n^(-1/5);      % h1

    Xbin_bt_xh1 = (X>=x_ep-h1);     % one for observation bigger than x-h1, 0 otherwise
    Xbin_lt_x = (X<x_ep);           % N- in p.16
    Xbin_bt_x = (X>=x_ep);          % N+
    Xbin_lt_xh1 = (X<=x_ep+h1);

    num_X_bt_x = sum(Xbin_bt_x); % # of observations of X bigger than x
    num_X_lt_x = sum(Xbin_lt_x); % # of observations of X less than x
    if (num_X_bt_x==0)||(num_X_lt_x==0)
        error('# of observations either bigger or less than x is zero')
    end

    Xbin_m = Xbin_bt_xh1.*Xbin_lt_x;    % index for X in [x-h,x]
    Xbin_p = Xbin_bt_x.*Xbin_lt_xh1;    % index for X in [x,x+h]
    Nh1_m = sum(Xbin_m);                % # of X in [x-h,x]
    Nh1_p = sum(Xbin_p);                % # of X in [x,x+h]
    if (Nh1_m==0)||(Nh1_p==0)||(Nh1_m==1)||(Nh1_p==1)
        error('# of observations is too few for step 1')
    end
    Nh1_sum = Nh1_m + Nh1_p;            % # of X in [x-h,x+h]
    
    % a. density estimate for X (kernel density estimation)
    % Naive esimate by IK
    f_c = Nh1_sum/(2*n*h1);              % f_c in (4.11)
    Yh1_bar_m = sum(Y.*Xbin_m)/Nh1_m;
    Yh1_bar_p = sum(Y.*Xbin_p)/Nh1_p;
        
    vy_m = sum(((Y-Yh1_bar_m).*Xbin_m).^2)/(Nh1_m-1);
    vy_p = sum(((Y-Yh1_bar_p).*Xbin_p).^2)/(Nh1_p-1);
    vc = (vy_m*(Nh1_m-1)+vy_p*(Nh1_p-1))/Nh1_sum; % vc in IK09
    
    %%%%%%%%%%%%%%%%%%%%%%%%
    % Step 2
    %%%%%%%%%%%%%%%%%%%%%%%%

    % 3rd order global polynomial regression
    Xx = X-x_ep;
    Xx2 = Xx.^2;
    Xx3 = Xx.*Xx2;
    Xx4 = Xx2.^2;
    nones = ones(n,1);
    data_mat = [Y nones Xx Xx2 Xx3 Xx4]; % include up to Xx4 for later use
    
    X_s2_p = X(Xbin_bt_x==1,:);
    X_s2_m = X(Xbin_lt_x==1,:);
    median_p = median(X_s2_p);
    median_m = median(X_s2_m);   
    middle = (X>=median_m)&(X<=median_p);
    data_step2 = data_mat(middle,:);
    X_step2 = [data_step2(:,2) (data_step2(:,3)>=0) data_step2(:,3:5)];
    Y_step2 = data_step2(:,1);
    beta_step2 = X_step2\Y_step2;
    gamma_4 = 6*beta_step2(5,1);
       
    % 2nd order LPR with uniform kernel

    h2_p = 3.56*(vc/f_c/gamma_4^2/num_X_bt_x)^(1/7);
    h2_m = 3.56*(vc/f_c/gamma_4^2/num_X_lt_x)^(1/7);
   
    Xbin_bet_xxh = (X>=x_ep).*(X<=x_ep+h2_p);     % index for X in [x, x+h2_p]
    pos_X_bet_xxh = find(Xbin_bet_xxh==1);
    data_trim_mat2_p = data_mat(pos_X_bet_xxh,:);
    YT_p = data_trim_mat2_p(:,1);
    T_p = data_trim_mat2_p(:,2:4);
    lambda_p_all = T_p\YT_p;
    m2c_p = 2*lambda_p_all(3,1);

    Xbin_bet_xhx = (X>=x_ep-h2_m).*(X<=x_ep);     % index for X in [x-h2_m, x]
    pos_X_bet_xhx = find(Xbin_bet_xhx==1);
    data_trim_mat2_m = data_mat(pos_X_bet_xhx,:);
    YT_m = data_trim_mat2_m(:,1);
    T_m = data_trim_mat2_m(:,2:4);
    lambda_m_all = T_m'*T_m\T_m'*YT_m;
    m2c_m = 2*lambda_m_all(3,1);

    %%%%%%%%%%%%%%%%%%%%%%%%
    % Step 3
    %%%%%%%%%%%%%%%%%%%%%%%%

    num_X_bet_xxh = sum(Xbin_bet_xxh);
    num_X_bet_xhx = sum(Xbin_bet_xhx);

    r_p = 720*vc/(num_X_bet_xxh*h2_p^4);
    r_m = 720*vc/(num_X_bet_xhx*h2_m^4);

    h3 = 3.4375*(2*vc/(n*f_c*((m2c_p - m2c_m)^2+r_p+r_m)))^(1/5);
    h_mat_1(ite,1) = h3;
    
    %%%%%%%%%%%%%%%%%%%%%%%%
    % Step 4 Get RD estimate
    %%%%%%%%%%%%%%%%%%%%%%%%
    [tau,delta_p,delta_m,~] = srdd(Y,X,x_ep,h3,h3);
    tau_bias_mat_1(ite,1) = tau - true_tau;
    delta_bias_mat_1(ite,:) = [delta_p-true_delta_p delta_m-true_delta_m];


    % Preparation
    
    pos_X_lt_x = find(Xbin_lt_x==1);    
    pos_X_bt_x = find(Xbin_bt_x==1);
    data_mat_p = data_mat(pos_X_bt_x,:); % data_mat only for positive X
    data_mat_m = data_mat(pos_X_lt_x,:); % data_mat only for negative X
    X_lt_x = data_mat_m(:,3);
    X_bt_x = data_mat_p(:,3);
    sorted_X_lt_x = sort(X_lt_x,'descend');     % descending order
    sorted_X_bt_x = sort(X_bt_x);               % ascending order
    sorted_X_lt_x_des_abs = abs(sort(X_lt_x,'descend'));
    options = optimset('Algorithm','interior-point','Display','off');



   
   
%    
% 3. CCT
%   

    output_cct = rddcct(X,Y,0);
    h_mat_3(ite,:) = output_cct.h;
    tau_bias_mat_3(ite,:) = output_cct.tau - true_tau;
    delta_bias_mat_3(ite,:) = output_cct.delta - [true_delta_p true_delta_m];
    b_mat_cct(ite,:) = output_cct.b;
 
    
%    
% 4. AI method with uniform kernel (MMSE)
%   (4th-order global fit => 3rd order LPR with h33 (m3)
%                       => 3rd order LPR with h23 (m2)
%                       => v_pilot from h_23
   % Step 1. Estimation of density and its first derivative
    % Kernel density estimation with normal scale bandwidth
    lambda = sqrt(var(X));
    a_pilot = 2.34*lambda/n^(1/5);
    f_k = density_est(X,x_ep,2,a_pilot); % kernel = 2, Epanechnikov

    h1_pilot = lambda*(105*16*sqrt(pi)/15/n)^(1/7);
    f1_pilot = density1d(X,x_ep,2,h1_pilot);
   
   % Step 2: Get pilot bandwidths to estimate derivatives
    % 4th order Polynomial regression 
    % extended version of page 13 of IK(11) to obtain ridge parameter 
    YT_pc = data_mat_p(:,1);
    TT_pc = data_mat_p(:,2:6);
    lambda_pc_4_4g = TT_pc\YT_pc;
    m41_4 = 24*lambda_pc_4_4g(5,1);
    
    YT_mc = data_mat_m(:,1);
    TT_mc = data_mat_m(:,2:6);
    lambda_mc_4_4g = TT_mc\YT_mc;
    m42_4 = 24*lambda_mc_4_4g(5,1);

    vy_p_plugin_4g = sum((YT_pc-TT_pc*lambda_pc_4_4g).^2)/(num_X_bt_x-5);
    vy_m_plugin_4g = sum((YT_mc-TT_mc*lambda_mc_4_4g).^2)/(num_X_lt_x-5);

   % Pilot bandiwidths for 3rd order LPR to estimate 3rd derivative with uniform kernel
    e4 = [0 0 0 0 1]';
    invTT_p = e4'*invpd(TT_pc'*TT_pc)*e4;
    invTT_m = e4'*invpd(TT_mc'*TT_mc)*e4;

    if kernel == 1
        h3_p = C33K_U*(vy_p_plugin_4g/f_k/m41_4^2)^(1/9)/num_X_bt_x^(1/9);
        h3_m = C33K_U*(vy_m_plugin_4g/f_k/m42_4^2)^(1/9)/num_X_lt_x^(1/9);
    else
        h3_p = C33K_T*(vy_p_plugin_4g/f_k/m41_4^2)^(1/9)/num_X_bt_x^(1/9);
        h3_m = C33K_T*(vy_m_plugin_4g/f_k/m42_4^2)^(1/9)/num_X_lt_x^(1/9);
    end
   % Pilot bandwidths3rd order LPR to estimate 2nd derivative with uniform kernel
    if kernel == 1
        h23_p = C23K_U*(vy_p_plugin_4g/f_k/m41_4^2)^(1/9)/num_X_bt_x^(1/9); % possible to use either v_pilot_24 or v_pilot_23
        h23_m = C23K_U*(vy_m_plugin_4g/f_k/m42_4^2)^(1/9)/num_X_lt_x^(1/9);
    else
        h23_p = C23K_T*(vy_p_plugin_4g/f_k/m41_4^2)^(1/9)/num_X_bt_x^(1/9); % possible to use either v_pilot_24 or v_pilot_23
        h23_m = C23K_T*(vy_m_plugin_4g/f_k/m42_4^2)^(1/9)/num_X_lt_x^(1/9);
    end

    h23_mat(ite,:) = [h23_p h23_m];
    h3_mat(ite,:) = [h3_p h3_m];

  % Step 3: Estimate the second and the third derivatives
   % 3rd order LPR for the third derivatives
    Xbin_bet_xxh = (X>=x_ep).*(X<=x_ep+h3_p);     % index for X in [x, x+h3_p]
    pos_X_bet_xxh = find(Xbin_bet_xxh==1);
    data_mat_4_p = data_mat(pos_X_bet_xxh,:);
    YT_p = data_mat_4_p(:,1);
    T_p = data_mat_4_p(:,2:5);
    Kvec_p = (1-abs(data_mat_4_p(:,3))/h3_p).*(abs(data_mat_4_p(:,3))<h3_p);
    W_p = diag(Kvec_p);
    if kernel == 1
        lambda_p_all_33_4 = T_p\YT_p;
    else
        lambda_p_all_33_4 = (T_p'*W_p*T_p)\(T_p'*W_p*YT_p);
    end
    
    m3c_p = 6*lambda_p_all_33_4(4,1);
    
    Xbin_bet_xhx = (X>=x_ep-h3_m).*(X<=x_ep);     % index for X in [x-h3_m, x]
    pos_X_bet_xhx = find(Xbin_bet_xhx==1);
    data_mat_4_m = data_mat(pos_X_bet_xhx,:);
    YT_m = data_mat_4_m(:,1);
    T_m = data_mat_4_m(:,2:5);
    Kvec_m = (1-abs(data_mat_4_m(:,3))/h3_m).*(abs(data_mat_4_m(:,3))<h3_m);
    W_m = diag(Kvec_m);
    if kernel == 1
        lambda_m_all_33_4 = T_m\YT_m;
    else
        lambda_m_all_33_4 = (T_m'*W_m*T_m)\(T_m'*W_m*YT_m);
    end
    m3c_m = 6*lambda_m_all_33_4(4,1);
    
    Xbin_bet_xxh = (X>=x_ep).*(X<=x_ep+h23_p);     % index for X in [x, x+h3_p]
    pos_X_bet_xxh = find(Xbin_bet_xxh==1);
    data_mat_4_p = data_mat(pos_X_bet_xxh,:);
    YT_p = data_mat_4_p(:,1);
    T_p = data_mat_4_p(:,2:5);
    Kvec_p = (1-abs(data_mat_4_p(:,3))/h23_p).*(abs(data_mat_4_p(:,3))<h23_p);
    W_p = diag(Kvec_p);
    if kernel == 1
        lambda_p_all_23 = T_p\YT_p;
    else
        lambda_p_all_23 = (T_p'*W_p*T_p)\(T_p'*W_p*YT_p);
    end
    m2c_p_2 = 2*lambda_p_all_23(3,1);    
    YT_p_hat = T_p*lambda_p_all_23;

    Xbin_bet_xhx = (X>=x_ep-h23_m).*(X<=x_ep);     % index for X in [x-h3_m, x]
    pos_X_bet_xhx = find(Xbin_bet_xhx==1);
    data_mat_4_m = data_mat(pos_X_bet_xhx,:);
    YT_m = data_mat_4_m(:,1);
    T_m = data_mat_4_m(:,2:5);
    Kvec_m = (1-abs(data_mat_4_m(:,3))/h23_m).*(abs(data_mat_4_m(:,3))<h23_m);
    W_m = diag(Kvec_m);
    if kernel == 1
        lambda_m_all_23 = T_m\YT_m;
    else
        lambda_m_all_23 = (T_m'*W_m*T_m)\(T_m'*W_m*YT_m);
    end
    m2c_m_2 = 2*lambda_m_all_23(3,1);

    YT_m_hat = T_m*lambda_m_all_23;

    vy_p_4_1 = sum((YT_p - YT_p_hat).^2)/(size(YT_p,1)-4);
    vy_m_4_1 = sum((YT_m - YT_m_hat).^2)/(size(YT_m,1)-4);

    vy_p_4_2 = (YT_p - YT_p_hat)'*W_p*(YT_p - YT_p_hat)/trace(W_p-W_p*T_p*invpd(T_p'*W_p*T_p)*T_p'*W_p);
    vy_m_4_2 = (YT_m - YT_m_hat)'*W_m*(YT_m - YT_m_hat)/trace(W_m-W_m*T_m*invpd(T_m'*W_m*T_m)*T_m'*W_m);

    v_pilot = [vy_p_4_2 vy_m_4_2];

    m2_pilot = [m2c_p_2 m2c_m_2];
    m3_pilot = [m3c_p m3c_m];
                                               
   % Step 4: Get bandwidths that minimize MMSE
    % Lower bound for AI search area
    NN_min_AI = 3;
    NN_p_AI = sorted_X_bt_x(NN_min_AI,1);
    NN_m_AI = sorted_X_lt_x_des_abs(NN_min_AI,1);
    h_lb = [NN_p_AI,NN_m_AI];
   % Upper bound for AI search area
    hu1 = sorted_X_bt_x(num_X_bt_x,1);
    hu2 = sorted_X_lt_x_des_abs(num_X_lt_x,1);
    h_ub = [hu1,hu2];

   % Compute residual
    res = zeros(n,1);
    Y_all = data_mat(:,1);
    Y_p_hat_all = data_mat(:,2:5)*lambda_p_all_23;
    res_p = Y_all - Y_p_hat_all;
    Y_m_hat_all = data_mat(:,2:5)*lambda_m_all_23;
    res_m = Y_all - Y_m_hat_all;

   Qn_value_2 = @(theta)Qn_AI_h_RDD(theta,f_k,f1_pilot,m2_pilot,m3_pilot,v_pilot,n);
   for j=1:num_ini
     h1_range = sorted_X_bt_x(num_X_bt_x,1) - sorted_X_bt_x(3,1);
     h2_range = sorted_X_lt_x_des_abs(num_X_lt_x,1) - sorted_X_lt_x_des_abs(3,1);
     h_ini_1 = sorted_X_bt_x(3,1) + 0.1*h1_range*j;
     h_ini_2 = sorted_X_lt_x_des_abs(3,1) + 0.1*h2_range*j;
     h_ini = [h_ini_1 h_ini_2];
     [theta_4, fval_4] = fmincon(Qn_value_2,h_ini,[],[],[],[],h_lb,h_ub,[],options);
     h_mat_local_4(j,:) = theta_4;
     min_val_local_4(j,1) = fval_4;
   end
   [~, min_index] = min(min_val_local_4);
   h_4 = h_mat_local_4(min_index,:);
   h_mat_4(ite,:) = h_4;

   [tau,delta_p,delta_m,~] = srdd(Y,X,x_ep,h_4(1),h_4(2));
   tau_bias_mat_4(ite,1) = tau-true_tau;
   delta_bias_mat_4(ite,:) = [delta_p-true_delta_p delta_m-true_delta_m];

   
   
%    
% 4. Independent selection
% 
    %
    
    X_step2 = [nones Xbin_bt_x Xx Xx2 Xx3];
    Y_step2 = data_step2(:,1);
    beta_step2 = X_step2\Y;
    gamma_4 = 6*beta_step2(5,1);
    
    % 2nd order LPR with uniform kernel
    h2_p = 3.56*(vy_p/f_c/gamma_4^2/num_X_bt_x)^(1/7);
    h2_m = 3.56*(vy_m/f_c/gamma_4^2/num_X_lt_x)^(1/7);
   
    Xbin_bet_xxh = (X>=x_ep).*(X<=x_ep+h2_p);     % index for X in [x, x+h2_p]
    data_trim_mat2_p = data_mat(Xbin_bet_xxh==1,:);
    YT_p = data_trim_mat2_p(:,1);
    T_p = data_trim_mat2_p(:,2:4);
    lambda_p_all = T_p\YT_p;
    m2c_p = 2*lambda_p_all(3,1);

    Xbin_bet_xhx = (X>=x_ep-h2_m).*(X<=x_ep);     % index for X in [x-h2_m, x]
    data_trim_mat2_m = data_mat(Xbin_bet_xhx==1,:);
    YT_m = data_trim_mat2_m(:,1);
    T_m = data_trim_mat2_m(:,2:4);
    lambda_m_all = T_m\YT_m;
    m2c_m = 2*lambda_m_all(3,1);
    
    % Get independent bandwidth
    h4p = 3.4375*(vy_p_4_1/(n*f_c*m2c_p^2))^(1/5);
    h4m = 3.4375*(vy_m_4_1/(n*f_c*m2c_m^2))^(1/5);
    h_mat_2(ite,:) = [h4p h4m];

   [tau,delta_p,delta_m,~] = srdd(Y,X,x_ep,h4p,h4m);
   tau_bias_mat_2(ite,1) = tau-true_tau;
   delta_bias_mat_2(ite,:) = [delta_p-true_delta_p delta_m-true_delta_m];
   
    
% end of M iteration
end

% Store bandwidths [MMSE_1 ID_1 IK CCT MMSE_0 IND_0]
h_mat = [h_mat_4(:,1) h_mat_2(:,1) h_mat_1 h_mat_3 h_mat_4(:,2) h_mat_2(:,2)];
h_mean = mean(h_mat);
h_sd = sqrt(var(h_mat));

tau_bias_mat = [tau_bias_mat_4 tau_bias_mat_2 tau_bias_mat_1 tau_bias_mat_3];
tau_bias = mean(tau_bias_mat);
tau_rmse = sqrt(mean(tau_bias_mat.^2));

delta_bias_mat = [delta_bias_mat_4 delta_bias_mat_2 delta_bias_mat_1 delta_bias_mat_3];
delta_bias = mean(delta_bias_mat);
delta_sd = sqrt(mean(delta_bias_mat.^2));
delta_result_mat(2*(D-1)+1:2*(D-1)+2,:) = [delta_bias; delta_sd];

if D==1
    save('SRD_tau_D1_16_n2k','tau_bias_mat')
    save('SRD_h_D1_16_n2k','h_mat')
    save('SRD_h23_D1_16_n2k','h23_mat')
    save('SRD_h3_D1_16_n2k','h3_mat')
    save('SRD_delta_D1_16_n2k','delta_bias_mat')
elseif D==2
    save('SRD_tau_D2_16_n2k','tau_bias_mat')
    save('SRD_h_D2_16_n2k','h_mat')
    save('SRD_h23_D2_16_n2k','h23_mat')
    save('SRD_h3_D2_16_n2k','h3_mat')
    save('SRD_delta_D2_16_n2k','delta_bias_mat')
elseif D==3
    save('SRD_tau_D3_16_n2k','tau_bias_mat')
    save('SRD_h_D3_16_n2k','h_mat')
    save('SRD_h23_D3_16_n2k','h23_mat')
    save('SRD_h3_D3_16_n2k','h3_mat')
    save('SRD_delta_D3_16_n2k','delta_bias_mat')
elseif D==4
    save('SRD_tau_D4_16_n2k','tau_bias_mat')
    save('SRD_h_D4_16_n2k','h_mat')
    save('SRD_h23_D4_16_n2k','h23_mat')
    save('SRD_h3_D4_16_n2k','h3_mat')
    save('SRD_delta_D4_16_n2k','delta_bias_mat')
    
else
    save('SRD_tau_D5_16_n2k','tau_bias_mat')
    save('SRD_h_D5_16_n2k','h_mat')
    save('SRD_h23_D5_16_n2k','h23_mat')
    save('SRD_h3_D5_16_n2k','h3_mat')
    save('SRD_delta_D5_16_n2k','delta_bias_mat')
end

D


% end of D loop
end

toc
beep
