function output = rddmmse_v01(X,Y,c)

%--------------------------------------------------------------------------
% rddmmse.m: compute the sharp regression discontinuity estimator
%--------------------------------------------------------------------------
%
% DESCRIPTION:
%   Estimate the sharp regression discontinuity estimator with the
%   bandwidths proposed by Arai and Ichimura (2015).
%
% REFERENCE:
%   Arai and Ichimura (2017). "Simultaneous Selection of Optimal Bandwidths 
%   for the Sharp Regression Discontinuity Estimator," Quantitative
%   Economics
% 
% INPUT ARGUMENTS
%   X: Imput vector (n by 1) of the forcing variable 
%   Y: Vector of the outcome variable (n by 1) 
%   c: Evaluation (discontinuity) point (scalar)
%
% OUTPUT ARGUMENTS:
%   tau:    sharp RD estimate
%   h:      bandwidths, the first element for the right of the cut-off
%                   the second element for the left of the cut-off
%   b2:     pilot bandwidths to estimate 2nd derivatives
%   b3:     pilot bandwidths to estimate 3rd derivatives
%       n:  the number of observations with nonzero weight,
%               the first element for the right of the cut-off,
%               the second element for the left of the cut-off
% 
% AUTHOR:
%   This code was written by Yoichi Arai.
%   I will appreciate any feedbacks.
%
% Note:
%   Optimization toolbox is necessary.
% 
%
% 
% This version: May, 2017
%--------------------------------------------------------------------------

% Check input arguments
if nargin < 3
    error('more input arguments needed.');
end

if nargin > 3
    error('Too many arguments.');
end

% Step 1: Estimation of density and its first derivative
 % Density Estimation
 n = size(X,1);
 lambda = sqrt(var(X));
 a_pilot = 2.34*lambda/n^(1/5);
 f_k = kdensity_est(X,c,a_pilot); % kernel = 2, Epanechnikov
 h1_pilot = lambda*(105*16*sqrt(pi)/15/n)^(1/7);
 f1_pilot = density1d_est(X,c,h1_pilot);
 
% Step 2: Get pilot bandwidths to estimate derivatives
 % 4th order Polynomial regression
 Xbin_lt_x = (X<c);           % binary indicator for observations less than x
 Xbin_bt_x = (X>=c);          % bigger than x
 num_X_bt_x = sum(Xbin_bt_x); % # of observations of X bigger than x, N+
 num_X_lt_x = sum(Xbin_lt_x); % # of observations of X less than x, N- in p.16
 Xx = X-c;
 Xx2 = Xx.^2;
 Xx3 = Xx.*Xx2;
 Xx4 = Xx2.^2;
 nones = ones(n,1);
 data_mat = [Y nones Xx Xx2 Xx3 Xx4];
 data_mat_p = data_mat(Xbin_bt_x==1,:); % data_mat only for positive X
 data_mat_m = data_mat(Xbin_lt_x==1,:); % data_mat only for negative X

 YT_pc = data_mat_p(:,1);
 TT_pc = data_mat_p(:,2:6);
 lambda_pc_4_4g = TT_pc\YT_pc;
 m41_4 = 24*lambda_pc_4_4g(5,1);
 
 YT_mc = data_mat_m(:,1);
 TT_mc = data_mat_m(:,2:6);
 lambda_mc_4_4g = TT_mc\YT_mc;
 m42_4 = 24*lambda_mc_4_4g(5,1);

 vy_p_plugin_4g = sum((YT_pc-TT_pc*lambda_pc_4_4g).^2)/(num_X_bt_x-5);
 vy_m_plugin_4g = sum((YT_mc-TT_mc*lambda_mc_4_4g).^2)/(num_X_lt_x-5);

 % Pilot bandwidths for 3rd order LPR to estimate 3rd derivative with uniform kernel
 C33K_T = Const_LPR(3,3);
 h3_p = C33K_T*(vy_p_plugin_4g/f_k/m41_4^2)^(1/9)/num_X_bt_x^(1/9);
 h3_m = C33K_T*(vy_m_plugin_4g/f_k/m42_4^2)^(1/9)/num_X_lt_x^(1/9);
 % Pilot bandwidths for 3rd order LPR to estimate 2nd derivative with uniform kernel
 C23K_T = Const_LPR(2,3);
 h23_p = C23K_T*(vy_p_plugin_4g/f_k/m41_4^2/num_X_bt_x)^(1/9); % possible to use either v_pilot_24 or v_pilot_23
 h23_m = C23K_T*(vy_m_plugin_4g/f_k/m42_4^2/num_X_lt_x)^(1/9);
 

% Step 3: Estimate the second and the third derivatives
 % 3rd order LPR for the third derivatives    
 Xbin_bet_xxh_h3 = Xbin_bt_x.*(X<=c+h3_p);     % index for X in [x, x+h3_p]
 data_mat_h3_p = data_mat(Xbin_bet_xxh_h3==1,:);
 YTh3_p = data_mat_h3_p(:,1);
 Th3_p = data_mat_h3_p(:,2:5);
 Kvec_p = (1-abs(data_mat_h3_p(:,3))/h3_p).*(abs(data_mat_h3_p(:,3))<h3_p);
 Wh3_p = diag(Kvec_p);
 lambda_p_all_33 = (Th3_p'*Wh3_p*Th3_p)\(Th3_p'*Wh3_p*YTh3_p);

 Xbin_bet_xhx_h3 = (X>=c-h3_m).*Xbin_lt_x;     % index for X in [x-h3_m, x]
 data_mat_h3_m = data_mat(Xbin_bet_xhx_h3==1,:);
 YTh3_m = data_mat_h3_m(:,1);
 Th3_m = data_mat_h3_m(:,2:5);
 Kvech3_m = (1-abs(data_mat_h3_m(:,3))/h3_m).*(abs(data_mat_h3_m(:,3))<h3_m);
 Wh3_m = diag(Kvech3_m);
 lambda_m_all_33 = (Th3_m'*Wh3_m*Th3_m)\(Th3_m'*Wh3_m*YTh3_m);

 m3c_p = 6*lambda_p_all_33(4,1);
 m3c_m = 6*lambda_m_all_33(4,1);

 % 3rd order LPR for the second derivatives
 Xbin_bet_xxh_h23 = Xbin_bt_x.*(X<=c+h23_p);     % index for X in [x, x+h3_p]
 data_mat_h23_p = data_mat(Xbin_bet_xxh_h23==1,:);
 YTh23_p = data_mat_h23_p(:,1);
 Th23_p = data_mat_h23_p(:,2:5);
 Kvech23_p = (1-abs(data_mat_h23_p(:,3))/h23_p).*(abs(data_mat_h23_p(:,3))<h23_p);
 Wh23_p = diag(Kvech23_p);
 lambda_p_all_23 = (Th23_p'*Wh23_p*Th23_p)\(Th23_p'*Wh23_p*YTh23_p);
 m2c_p_2 = 2*lambda_p_all_23(3,1);

 YTh23_p_hat = Th23_p*lambda_p_all_23;
 vy_p_4_2 = (YTh23_p - YTh23_p_hat)'*Wh23_p*(YTh23_p - YTh23_p_hat)/trace(Wh23_p-Wh23_p*Th23_p*invpd(Th23_p'*Wh23_p*Th23_p)*Th23_p'*Wh23_p);

 Xbin_bet_xhx_h23 = (X>=c-h23_m).*Xbin_lt_x;     % index for X in [x-h3_m, x]
 data_mat_h23_m = data_mat(Xbin_bet_xhx_h23==1,:);
 YTh23_m = data_mat_h23_m(:,1);
 Th23_m = data_mat_h23_m(:,2:5);
 Kvech23_m = (1-abs(data_mat_h23_m(:,3))/h23_m).*(abs(data_mat_h23_m(:,3))<h23_m);
 Wh23_m = diag(Kvech23_m);
 lambda_m_all_23 = (Th23_m'*Wh23_m*Th23_m)\(Th23_m'*Wh23_m*YTh23_m);
 m2c_m_2 = 2*lambda_m_all_23(3,1);

 YTh23_m_hat = Th23_m*lambda_m_all_23;
 vy_m_4_2 = (YTh23_m - YTh23_m_hat)'*Wh23_m*(YTh23_m - YTh23_m_hat)/trace(Wh23_m-Wh23_m*Th23_m*invpd(Th23_m'*Wh23_m*Th23_m)*Th23_m'*Wh23_m);

 m2_pilot = [m2c_p_2 m2c_m_2];
 m3_pilot = [m3c_p m3c_m];
 v_pilot = [vy_p_4_2 vy_m_4_2];
 
% Step 4: Get bandwidths that minimize MMSE
 options = optimset('Algorithm','interior-point','Display','off');
 Qn_value_2 = @(theta)Qn(theta,f_k,f1_pilot,m2_pilot,m3_pilot,v_pilot,n);

 % Lower bound for AI search area
 NN_min_AI = 3;
 X_lt_x = data_mat_m(:,3);
 X_bt_x = data_mat_p(:,3);
 sorted_X_bt_x = sort(X_bt_x);               % ascending order
 sorted_X_lt_x_des_abs = abs(sort(X_lt_x,'descend'));
 NN_p_AI = sorted_X_bt_x(NN_min_AI,1);
 NN_m_AI = sorted_X_lt_x_des_abs(NN_min_AI,1);
 h_lb = [NN_p_AI,NN_m_AI];
 % Upper bound for AI search area
 hu1 = sorted_X_bt_x(num_X_bt_x,1);
 hu2 = sorted_X_lt_x_des_abs(num_X_lt_x,1);
 h_ub = [hu1,hu2];

 h_mat_local = zeros(10,2);
 min_val = zeros(10,1);
 h1_range = sorted_X_bt_x(num_X_bt_x,1) - sorted_X_bt_x(3,1);
 h2_range = sorted_X_lt_x_des_abs(num_X_lt_x,1) - sorted_X_lt_x_des_abs(3,1);
 
 
 for k=1:10
 for j=1:10
     h_ini_1 = sorted_X_bt_x(3,1) + 0.1*h1_range*j;
     h_ini_2 = sorted_X_lt_x_des_abs(3,1) + 0.1*h2_range*k;
     h_ini = [h_ini_1 h_ini_2];
     [theta_2, fval_2] = fmincon(Qn_value_2,h_ini,[],[],[],[],h_lb,h_ub,[],options);
     h_mat_local(10*(k-1)+j,:) = theta_2;
     min_val(10*(k-1)+j,1) = fval_2;
     
 end
 end
 [~, min_index] = min(min_val);
 h_opt = h_mat_local(min_index,:);
 
% Compute SRDD estimate
Xbin_bt_xh_opt = (X >= c-h_opt(2));
Xbin_lt_xh_opt = (X <= c+h_opt(1));
Xbin_bet_xxh_opt = Xbin_bt_x.*Xbin_lt_xh_opt;    % index for X in [x,x+h]
Xbin_bet_xhx_opt = Xbin_bt_xh_opt.*Xbin_lt_x;    % index for X in [x-h,x]
n_p = sum(Xbin_bet_xxh_opt);
n_m = sum(Xbin_bet_xhx_opt);

data_trim_mat_p_opt = data_mat.*(Xbin_bet_xxh_opt*ones(1,6));
T_p_opt = data_trim_mat_p_opt(:,2:3);
Kvec_p_opt = (1-abs(data_trim_mat_p_opt(:,3))/h_opt(1)).*(abs(data_trim_mat_p_opt(:,3))<h_opt(1));
W_p_opt = diag(Kvec_p_opt.*Xbin_bet_xxh_opt);
YT_p_opt = data_trim_mat_p_opt(:,1);
lambda_p_all_opt = (T_p_opt'*W_p_opt*T_p_opt)\(T_p_opt'*W_p_opt*YT_p_opt);
delta_p_opt = lambda_p_all_opt(1,1);

data_trim_mat_m_opt = data_mat.*(Xbin_bet_xhx_opt*ones(1,6));
T_m_opt = data_trim_mat_m_opt(:,2:3);
Kvec_m_opt = (1-abs(data_trim_mat_m_opt(:,3))/h_opt(2)).*(abs(data_trim_mat_m_opt(:,3))<h_opt(2));
W_m_opt = diag(Kvec_m_opt.*Xbin_bet_xhx_opt);
YT_m_opt = data_trim_mat_m_opt(:,1);
lambda_m_all_opt = (T_m_opt'*W_m_opt*T_m_opt)\(T_m_opt'*W_m_opt*YT_m_opt);
delta_m_opt = lambda_m_all_opt(1,1);


tau = delta_p_opt - delta_m_opt;

 

output.tau = tau;
output.h = h_opt;
output.b2 = [h23_p h23_m]; 
output.b3 = [h3_p h3_m]; 
output.n = [n_p n_m];

%------------------------------------------------------------------
% End function rddmmse
%------------------------------------------------------------------

function fhat = kdensity_est(X,c,h)

% Purpose: estimate density at a point with Epanechnikov kernel
% --------------------------------------------
% Usage: results = density_est
%       X = a column vector of scalar random variables
%       c = point to estimate
%       h = bandwidth
% --------------------------------------------

nobs = size(X,1);
%fhat = 0;
A_h = (X-c)/h;
ExpA_h = 3*sum((abs(A_h)<1).*(1-A_h.^2))/4;
fhat = ExpA_h/nobs/h;

%------------------------------------------------------------------
% End private function density_est
%------------------------------------------------------------------

function f1hat = density1d_est(X,c,h)

% Purpose: estimate 1st derivative density at a point
% --------------------------------------------
% Usage: results = density_est
%       X = a column vector of scalar random variables
%       c = point to estimate
%       h = bandwidth
% --------------------------------------------

nobs = size(X,1);
A_h = (c-X)/h;
B_h = A_h.^2;
k2 =1/5;
const = 3/4;
ExpA_h = (-1)*const*sum(A_h.*(1-B_h).*(abs(A_h)<1))/k2;
f1hat = ExpA_h/nobs/h^2;

%------------------------------------------------------------------
% End private function density1d_est
%------------------------------------------------------------------

function [C_vp] = Const_LPR(v,p)

% S = (\mu_{j+k}), 0<=j,k<=p, defined in FG(96,p61), p=6
S6 = [1/2 1/6 1/12 1/20 1/30 1/42 1/56; 
       1/6 1/12 1/20 1/30 1/42 1/56 1/72;
       1/12 1/20 1/30 1/42 1/56 1/72 1/90; 
       1/20 1/30 1/42 1/56 1/72 1/90 1/110;
       1/30 1/42 1/56 1/72 1/90 1/110 1/132; 
       1/42 1/56 1/72 1/90 1/110 1/132 1/156;
       1/56 1/72 1/90 1/110 1/132 1/156 1/182];
% Sstar = (\nu_{j+k}), 0<=j,k<=p, defined in FG(96,p61), p=6
K6 = [1/3 1/12 1/30 1/60 1/105 1/168 1/252;
        1/12 1/30 1/60 1/105 1/168 1/252 1/360;
        1/30 1/60 1/105 1/168 1/252 1/360 1/495;
        1/60 1/105 1/168 1/252 1/360 1/495 1/660;
        1/105 1/168 1/252 1/360 1/495 1/660 1/858;
        1/168 1/252 1/360 1/495 1/660 1/858 1/1092;
        1/252 1/360 1/495 1/660 1/858 1/1092 1/1365];
        
% Vector of \mu_j,
% [\mu_1, \mu_2, ..., \mu_{13}]
mu_vec = [1/6 1/12 1/20 1/30 1/42 1/56 1/72 1/90 1/110 1/132 1/156 1/182 1/210];

Sinv_p = invpd(S6(1:p+1,1:p+1));

% int t^{p+1}K_{v,p}^*(t)dt
int_tp_K = Sinv_p(v+1,:)*mu_vec(1,(p+1):(2*p+1))';

% \int_K_{v,p}^{*2}(t)dt
int_K2 = Sinv_p(v+1,:)*K6(1:p+1,1:p+1)*Sinv_p(v+1,:)';

C_vp = ((factorial(p+1)^2*(2*v+1)*int_K2)/(2*(p+1-v)*int_tp_K^2))^(1/(2*p+3));

%------------------------------------------------------------------
% End private function Const_LPR
%------------------------------------------------------------------

function [Qn] = Qn(h,f_pilot,f1_pilot,m2_pilot,m3_pilot,v_pilot,n)

% matrix for constants depending on kernel
% [b1 c1 c2 v]
k_const = [-0.1 -0.1 -0.08 24/5];

b2 = zeros(1,2);
m2foverf12 = zeros(1,2);
for j=1:2
    m2foverf12(j) = m2_pilot(j)*f1_pilot/f_pilot/2;
    b2(j) = (-1)^(j+1)*(k_const(2)*(m2foverf12(j) + m3_pilot(j)/6)...   
         - k_const(3)*m2foverf12(j));
end
    
Qn = k_const(1)^2*(m2_pilot(1)*h(1)^2 - m2_pilot(2)*h(2)^2)^2/4 ...
+(b2(1)*h(1)^3 - b2(2)*h(2)^3)^2 ...
+ k_const(4)*(v_pilot(1)/h(1) + v_pilot(2)/h(2))/f_pilot/n;

%------------------------------------------------------------------
% End private function Qn
%------------------------------------------------------------------


