function output = rddci_mmse_v01(X,Y,c,h_opt,h23)

%--------------------------------------------------------------------------
% rddci.m: compute the CI for sharp regression discontinuity estimator
%           and its related quantities
%--------------------------------------------------------------------------
%
% DESCRIPTION:
%   Estimate the sharp regression discontinuity estimator with the
%   bandwidths proposed by Arai and Ichimura (2017).
%   Compute the robust and undersmoothing versions of the confidence
%   intervals
%
% REFERENCE:
%   Arai and Ichimura (2017). "Simultaneous Selection of Optimal Bandwidths 
%   for the Sharp Regression Discontinuity Estimator," Quantitative
%   Economics
% 
% INPUT ARGUMENTS
%   X:      Imput vector (n by 1) of the forcing variable 
%   Y:      Vector of the outcome variable (n by 1) 
%   c:      Evaluation (discontinuity) point (scalar)
%   h_opt:  Main bandwidths
%   h23:     Pilot bandwidths to estimate 2nd derivatives
%   kernel = 1; % 1: triangular, 2: uniform
%   
%
% OUTPUT ARGUMENTS:
%   h:      Main bandwidths, the first element for the right of the cut-off
%   num:    Number of observations used, the first element for the right of the cut-off 
%   tau:    Sharp RD estimate
%   se_r:   CCT-type robust standard error
%   tval_r: bias-corrected t-statistic based on se_r
%   pval:   p-value based on tval_r
%   ci_r:   CCT-type robust confidence interval
%   ci_us:  Confidence interval by undersmoothing
% 
% AUTHOR:
%   This code was written by Yoichi Arai.
%   I will appreciate any feedbacks.
%
% Note:
%   Optimization toolbox is necessary.
% 
%
% 
% This version: May, 2017
%--------------------------------------------------------------------------

h_mmse = h_opt;
b2 = h_opt;
b3 = h_opt;


% Compute SRDD estimate
n = size(X,1);
Xx = X-c;
Xx2 = Xx.^2;
Xx3 = Xx.*Xx2;
Xx4 = Xx2.^2;
nones = ones(n,1);
data_mat = [Y nones Xx Xx2 Xx3 Xx4];
Xbin_lt_x = (X<c);           % binary indicator for observations less than x
Xbin_bt_x = (X>=c);          % bigger than x
data_mat_p = data_mat(Xbin_bt_x==1,:); % data_mat only for positive X
data_mat_m = data_mat(Xbin_lt_x==1,:); % data_mat only for negative X
yp = data_mat_p(:,1);
ym = data_mat_m(:,1);

% Check the sign of the second derivatives
h23_p = h23(1);
h23_m = h23(2);

Xbin_bet_xxh_h23 = Xbin_bt_x.*(X<=c+h23_p);     % index for X in [x, x+h3_p]
data_mat_h23_p = data_mat(Xbin_bet_xxh_h23==1,:);
YTh23_p = data_mat_h23_p(:,1);
Th23_p = data_mat_h23_p(:,2:5);
Kvech23_p = (1-abs(data_mat_h23_p(:,3))/h23_p).*(abs(data_mat_h23_p(:,3))<h23_p);
Wh23_p = diag(Kvech23_p);
lambda_p_all_23 = (Th23_p'*Wh23_p*Th23_p)\(Th23_p'*Wh23_p*YTh23_p);
m2c_p_2 = 2*lambda_p_all_23(3,1);
Xbin_bet_xhx_h23 = (X>=c-h23_m).*Xbin_lt_x;     % index for X in [x-h3_m, x]

data_mat_h23_m = data_mat(Xbin_bet_xhx_h23==1,:);
YTh23_m = data_mat_h23_m(:,1);
Th23_m = data_mat_h23_m(:,2:5);
Kvech23_m = (1-abs(data_mat_h23_m(:,3))/h23_m).*(abs(data_mat_h23_m(:,3))<h23_m);
Wh23_m = diag(Kvech23_m);
lambda_m_all_23 = (Th23_m'*Wh23_m*Th23_m)\(Th23_m'*Wh23_m*YTh23_m);
m2c_m_2 = 2*lambda_m_all_23(3,1);

sign_m2 = (m2c_p_2*m2c_m_2>=0);

if sign_m2==1
    h_opt = n^(-1/25)*h_opt;
else
end

% Constants
% [b1 xi1 xi2 v]
k_const = [-0.1 -0.1 -0.08 24/5];

% Estimation of density and its first derivative
lambda = sqrt(var(Xx));
a_pilot = 2.34*lambda/n^(1/5);
f_k = kdensity_est(Xx,0,a_pilot);
h1_pilot = lambda*(105*16*sqrt(pi)/15/n)^(1/7);
f1_pilot = density1d_est(Xx,0,h1_pilot);

% Compute tau and V1
e0 = [1 0]';
h_vec_p = [1 1/h_opt(1) 1/h_opt(1)^2 1/h_opt(1)^3];
h_vec_m = [1 1/h_opt(2) 1/h_opt(2)^2 1/h_opt(2)^3];
H_p2 = diag(h_vec_p(1:2));
H_m2 = diag(h_vec_m(1:2));
H_p3 = diag(h_vec_p);
H_m3 = diag(h_vec_m);
X3h_p = data_mat_p(:,2:5)*H_p3;
X3h_m = data_mat_m(:,2:5)*H_m3;
Kvec_h_p = (1-abs(data_mat_p(:,3))/h_opt(1)).*(abs(data_mat_p(:,3))<h_opt(1));
Kvec_h_m = (1-abs(data_mat_m(:,3))/h_opt(2)).*(abs(data_mat_m(:,3))<h_opt(2));
Kind_h_p = find(Kvec_h_p ~= 0);
Kind_h_m = find(Kvec_h_m ~= 0);
yh_p = yp(Kind_h_p,1);
yh_m = ym(Kind_h_m,1);
num = [size(yh_p,1) size(yh_m,1)];
X3h_p2 = X3h_p(Kind_h_p,1:2);
X3h_m2 = X3h_m(Kind_h_m,1:2);
Kvec_h_p2 = Kvec_h_p(Kind_h_p,1);
Kvec_h_m2 = Kvec_h_m(Kind_h_m,1);
Wh_p = diag(Kvec_h_p2);
Wh_m = diag(Kvec_h_m2);
Gamma_h_p = X3h_p2'*Wh_p*X3h_p2/n;
Gamma_h_m = X3h_m2'*Wh_m*X3h_m2/n;
invGamma_h_p = invpd(Gamma_h_p);
invGamma_h_m = invpd(Gamma_h_m);
delta_p_opt = e0'*invGamma_h_p*X3h_p2'*Wh_p*yh_p/n;
delta_m_opt = e0'*invGamma_h_m*X3h_m2'*Wh_m*yh_m/n;
beta_p = invGamma_h_p*X3h_p2'*Wh_p*yh_p/n;
beta_m = invGamma_h_m*X3h_m2'*Wh_m*yh_m/n;
YT_p_hat = X3h_p2*beta_p;
s2_p = (yh_p - YT_p_hat)'*Wh_p*(yh_p - YT_p_hat)/trace(Wh_p-Wh_p*X3h_p2*invpd(X3h_p2'*Wh_p*X3h_p2)*X3h_p2'*Wh_p);
YT_m_hat = X3h_m2*beta_m;
s2_m = (yh_m - YT_m_hat)'*Wh_m*(yh_m - YT_m_hat)/trace(Wh_m-Wh_m*X3h_m2*invpd(X3h_m2'*Wh_m*X3h_m2)*X3h_m2'*Wh_m);

Psi_h_p = s2_p*X3h_p2'*Wh_p*Wh_p*X3h_p2/n;
Psi_h_m = s2_m*X3h_m2'*Wh_m*Wh_m*X3h_m2/n;
Vh_p = (e0'*invGamma_h_p*Psi_h_p*invGamma_h_p*e0)/n;
Vh_m = (e0'*invGamma_h_m*Psi_h_m*invGamma_h_m*e0)/n;
V1 = Vh_p + Vh_m;

% Compute V2
e2 = [0 0 1 0]';
b2_vec_p = [1 1/b2(1) 1/b2(1)^2 1/b2(1)^3];
b2_vec_m = [1 1/b2(2) 1/b2(2)^2 1/b2(2)^3];
Hb2_p = diag(b2_vec_p);
Hb2_m = diag(b2_vec_m);
X3b2_p = data_mat_p(:,2:5)*Hb2_p;
X3b2_m = data_mat_m(:,2:5)*Hb2_m;
Kvec_b2_p = (1-abs(data_mat_p(:,3))/b2(1)).*(abs(data_mat_p(:,3))<b2(1));
Kvec_b2_m = (1-abs(data_mat_m(:,3))/b2(2)).*(abs(data_mat_m(:,3))<b2(2));
Kind_b2_p = find(Kvec_b2_p ~= 0);
Kind_b2_m = find(Kvec_b2_m ~= 0);
yb2_p = yp(Kind_b2_p,1);
yb2_m = ym(Kind_b2_m,1);
X3b2_p2 = X3b2_p(Kind_b2_p,:);
X3b2_m2 = X3b2_m(Kind_b2_m,:);
Kvec_b2_p2 = Kvec_b2_p(Kind_b2_p,1);
Kvec_b2_m2 = Kvec_b2_m(Kind_b2_m,1);
Wb2_p = diag(Kvec_b2_p2);
Wb2_m = diag(Kvec_b2_m2);
Gamma_b2_p = X3b2_p2'*Wb2_p*X3b2_p2/n;
Gamma_b2_m = X3b2_m2'*Wb2_m*X3b2_m2/n;
invGamma_b2_p = invpd(Gamma_b2_p);
invGamma_b2_m = invpd(Gamma_b2_m);
m2_p = 2*e2'*Hb2_p*invGamma_b2_p*X3b2_p2'*Wb2_p*yb2_p/n;
m2_m = 2*e2'*Hb2_m*invGamma_b2_m*X3b2_m2'*Wb2_m*yb2_m/n;

Psi_b2_p = s2_p*X3b2_p2'*Wb2_p*Wb2_p*X3b2_p2/n;
Psi_b2_m = s2_m*X3b2_m2'*Wb2_m*Wb2_m*X3b2_m2/n;
V23_b_p = b2(1)^(-4)*(e2'*invGamma_b2_p*Psi_b2_p*invGamma_b2_p*e2)/n;
V23_b_m = b2(2)^(-4)*(e2'*invGamma_b2_m*Psi_b2_m*invGamma_b2_m*e2)/n;
Theta2_h_p = X3h_p2'*Wh_p*X3h_p(Kind_h_p,3)/n;
Theta2_h_m = X3h_m2'*Wh_m*X3h_m(Kind_h_m,3)/n;
B012_p = e0'*H_p2*invGamma_h_p*Theta2_h_p;
B012_m = e0'*H_m2*invGamma_h_m*Theta2_h_m;
V2_p = h_opt(1)^4*V23_b_p*k_const(1)^2;
V2_m = h_opt(2)^4*V23_b_m*k_const(1)^2;
V2 = V2_p + V2_m;

% Compute V3_1
B31_const = (k_const(2)-k_const(3))*f1_pilot/f_k;
V31_p = h_opt(1)^6*V23_b_p*B31_const^2;
V31_m = h_opt(2)^6*V23_b_m*B31_const^2;
V31 = V31_p + V31_m;

% Compute V3_2
e3 = [0 0 0 1]';
b3_vec_p = [1 1/b3(1) 1/b3(1)^2 1/b3(1)^3];
b3_vec_m = [1 1/b3(2) 1/b3(2)^2 1/b3(2)^3];
Hb3_p = diag(b3_vec_p);
Hb3_m = diag(b3_vec_m);
X3b3_p = data_mat_p(:,2:5)*Hb3_p;
X3b3_m = data_mat_m(:,2:5)*Hb3_m;
Kvec_b3_p = (1-abs(data_mat_p(:,3))/b3(1)).*(abs(data_mat_p(:,3))<b3(1));
Kvec_b3_m = (1-abs(data_mat_m(:,3))/b3(2)).*(abs(data_mat_m(:,3))<b3(2));
Kind_b3_p = find(Kvec_b3_p ~= 0);
Kind_b3_m = find(Kvec_b3_m ~= 0);
yb3_p = yp(Kind_b3_p,1);
yb3_m = ym(Kind_b3_m,1);
X3b3_p2 = X3b3_p(Kind_b3_p,:);
X3b3_m2 = X3b3_m(Kind_b3_m,:);
Kvec_b3_p2 = Kvec_b3_p(Kind_b3_p,1);
Kvec_b3_m2 = Kvec_b3_m(Kind_b3_m,1);
Wb3_p = diag(Kvec_b3_p2);
Wb3_m = diag(Kvec_b3_m2);
Gamma_b3_p = X3b3_p2'*Wb3_p*X3b3_p2/n;
Gamma_b3_m = X3b3_m2'*Wb3_m*X3b3_m2/n;
invGamma_b3_p = invpd(Gamma_b3_p);
invGamma_b3_m = invpd(Gamma_b3_m);
m3_p = 6*e3'*Hb3_p*invGamma_b3_p*X3b3_p2'*Wb3_p*yb3_p/n;
m3_m = 6*e3'*Hb3_m*invGamma_b3_m*X3b3_m2'*Wb3_m*yb3_m/n;

Psi_b3_p = s2_p*X3b3_p2'*Wb3_p*Wb3_p*X3b3_p2/n;
Psi_b3_m = s2_m*X3b3_m2'*Wb3_m*Wb3_m*X3b3_m2/n;
V33_b_p = b3(1)^(-6)*(e3'*invGamma_b3_p*Psi_b3_p*invGamma_b3_p*e3)/n;
V33_b_m = b3(2)^(-6)*(e3'*invGamma_b3_m*Psi_b3_m*invGamma_b3_m*e3)/n;
Theta3_h_p = X3h_p2'*Wh_p*X3h_p(Kind_h_p,4)/n;
Theta3_h_m = X3h_m2'*Wh_m*X3h_m(Kind_h_m,4)/n;
B013_p = e0'*H_p2*invGamma_h_p*Theta3_h_p;
B013_m = e0'*H_m2*invGamma_h_m*Theta3_h_m;

V32_p = h_opt(1)^6*V33_b_p*k_const(2)^2;
V32_m = h_opt(2)^6*V33_b_m*k_const(2)^2;
V32 = V32_p + V32_m;

% Compute C12
if h_opt(1) <= b2(1)
    Kvec_b2_p3 = Kvec_b2_p(Kind_h_p,1);
    Wb2_p2 = diag(Kvec_b2_p3);
    X3b2_p3 = X3b2_p(Kind_h_p,:); 
    Psi13_hb2_p = s2_p*X3h_p2'*Wh_p*Wb2_p2*X3b2_p3/n;  
else
    Kvec_h_p2 = Kvec_h_p(Kind_b2_p,1);
    Wh_p2 = diag(Kvec_h_p2);
    X3h_p3 = X3h_p(Kind_b2_p,1:2);
    Psi13_hb2_p = s2_p*X3h_p3'*Wh_p2*Wb2_p*X3b2_p2/n;
end
      
if h_opt(2) <= b2(2)   
    Kvec_b2_m3 = Kvec_b2_m(Kind_h_m,1);
    Wb2_m2 = diag(Kvec_b2_m3);    
    X3b2_m3 = X3b2_m(Kind_h_m,:);
    Psi13_hb2_m = s2_m*X3h_m2'*Wh_m*Wb2_m2*X3b2_m3/n;
else
    Kvec_h_m2 = Kvec_h_m(Kind_b2_m,1);
    Wh_m2 = diag(Kvec_h_m2);
    X3h_m3 = X3h_m(Kind_b2_m,1:2);
    Psi13_hb2_m = s2_m*X3h_m3'*Wh_m2*Wb2_m*X3b2_m2/n;
end  
    
D12_hb2_p = b2(1)^(-2)*e0'*invGamma_h_p*Psi13_hb2_p*invGamma_b2_p*e2/n;
D12_hb2_m = b2(2)^(-2)*e0'*invGamma_h_m*Psi13_hb2_m*invGamma_b2_m*e2/n;
C12_p = h_opt(1)^2*D12_hb2_p*k_const(1);
C12_m = h_opt(2)^2*D12_hb2_m*k_const(1);

% Compute C13_1
C131_p = h_opt(1)^3*D12_hb2_p*B31_const;
C131_m = h_opt(2)^3*D12_hb2_m*B31_const;

% Compute C13_2
if h_opt(1) <= b3(1)
    Kvec_b3_p13 = Kvec_b3_p(Kind_h_p,1);
    Wb3_p13 = diag(Kvec_b3_p13);
    X3b3_p13 = X3b3_p(Kind_h_p,:); 
    Psi13_hb3_p = s2_p*X3h_p2'*Wh_p*Wb3_p13*X3b3_p13/n;  
else
    Kvec_h_p2 = Kvec_h_p(Kind_b3_p,1);
    Wh_p13 = diag(Kvec_h_p2);
    X3h_p13 = X3h_p(Kind_b3_p,1:2);
    Psi13_hb3_p = s2_p*X3h_p13'*Wh_p13*Wb3_p*X3b3_p2/n;
end
      
if h_opt(2) <= b3(2)   
    Kvec_b3_m13 = Kvec_b3_m(Kind_h_m,1);
    Wb3_m13 = diag(Kvec_b3_m13);    
    X3b3_m13 = X3b3_m(Kind_h_m,:);
    Psi13_hb3_m = s2_m*X3h_m2'*Wh_m*Wb3_m13*X3b3_m13/n;
else
    Kvec_h_m2 = Kvec_h_m(Kind_b3_m,1);
    Wh_m13 = diag(Kvec_h_m2);
    X3h_m13 = X3h_m(Kind_b3_m,1:2);
    Psi13_hb3_m = s2_m*X3h_m13'*Wh_m13*Wb3_m*X3b3_m2/n;
end  
    
D13_hb3_p = b3(1)^(-3)*e0'*invGamma_h_p*Psi13_hb3_p*invGamma_b3_p*e3/n;
D13_hb3_m = b3(2)^(-3)*e0'*invGamma_h_m*Psi13_hb3_m*invGamma_b3_m*e3/n;
C132_p = h_opt(1)^3*D13_hb3_p*k_const(2);
C132_m = h_opt(2)^3*D13_hb3_m*k_const(2);

% Compute C23_1
C231_p = h_opt(1)^5*V23_b_p*B012_p*k_const(2)*B31_const;
C231_m = h_opt(2)^5*V23_b_m*B012_m*k_const(2)*B31_const;

% Compute C23_2
if b2(1) <= b3(1)
    Kvec_b3_p23 = Kvec_b3_p(Kind_b2_p,1);
    Wb3_p23 = diag(Kvec_b3_p23);
    X3b3_p23 = X3b3_p(Kind_b2_p,:); 
    Psi33_b23_p = s2_p*X3b2_p2'*Wb2_p*Wb3_p23*X3b3_p23/n;
else
    Kvec_b2_p23 = Kvec_h_p(Kind_b3_p,1);
    Wb2_p23 = diag(Kvec_b2_p23);
    X3b2_p23 = X3b2_p(Kind_b3_p,:);
    Psi33_b23_p = s2_p*X3b2_p23'*Wb2_p23*Wb3_p*X3b3_p2/n;
end
      
if b2(2) <= b3(2)   
    Kvec_b3_m23 = Kvec_b3_m(Kind_b2_m,1);
    Wb3_m23 = diag(Kvec_b3_m23);    
    X3b3_m23 = X3b3_m(Kind_b2_m,:);
    Psi33_b23_m = s2_m*X3b2_m2'*Wb2_m*Wb3_m23*X3b3_m23/n;
else
    Kvec_b2_m23 = Kvec_b2_m(Kind_b3_m,1);
    Wb2_m23 = diag(Kvec_b2_m23);
    X3b2_m23 = X3b2_m(Kind_b3_m,:);
    Psi33_b23_m = s2_m*X3b2_m23'*Wb2_m23*Wb3_m*X3b3_m2/n;
end  

D23_hb23_p = b2(1)^(-2)*b3(1)^(-3)*e2'*invGamma_b2_p*Psi33_b23_p*invGamma_b3_p*e3/n;
D23_hb23_m = b2(2)^(-2)*b3(2)^(-3)*e2'*invGamma_b2_m*Psi33_b23_m*invGamma_b3_m*e3/n;
C232_p = h_opt(1)^5*D23_hb23_p*k_const(1)*k_const(2);
C232_m = h_opt(2)^5*D23_hb23_m*k_const(1)*k_const(2);

% Compute C33
C33_p = h_opt(1)^6*D23_hb23_p*k_const(2)*B31_const;
C33_m = h_opt(2)^6*D23_hb23_m*k_const(2)*B31_const;

% Compute the robust variance
%Vrbc_cct = V1 + V2 - 2*C12_p - 2*C12_m;
Vrbc = V1 + V2 + V31 + V32 - 2*C12_p - 2*C12_m - 2*C131_p + 2*C131_m - 2*C132_p + 2*C132_m ...
    + 2*C231_p - 2*C231_m + 2*C232_p - 2*C232_m + 2*C33_p + 2*C33_m;

tau = delta_p_opt - delta_m_opt;
se_r = sqrt(Vrbc);

bias1_p = h_opt(1)^2*m2_p*k_const(1)/2;
bias1_m = h_opt(2)^2*m2_m*k_const(1)/2;
bias21_p = h_opt(1)^3*m2_p*B31_const/2;
bias21_m = h_opt(2)^3*m2_m*B31_const/2;
bias22_p = h_opt(1)^3*m3_p*k_const(2)/6;
bias22_m = h_opt(2)^3*m3_m*k_const(2)/6;

b11_vec_opt = bias1_p + bias21_p + bias22_p;
b12_vec_opt = bias1_m - bias21_m - bias22_m;

bias_opt = b11_vec_opt-b12_vec_opt;

% Compute the undersmoothed CI
k=6.5;
hus = n^(-1/k)*h_mmse;
n_lt_h_p = sum(data_mat_p(:,3)<hus(1));
n_lt_h_m = sum(abs(data_mat_p(:,3))<hus(2));

while n_lt_h_p<=2 || n_lt_h_m<=2
    k = k+1/2;
    hus = n^(-1/k)*h_mmse;
    n_lt_h_p = sum(data_mat_p(:,3)<hus(1));
    n_lt_h_m = sum(abs(data_mat_p(:,3))<hus(2));
end


% Compute tau and V1
e0 = [1 0]';
h_vec_p = [1 1/hus(1) 1/hus(1)^2 1/hus(1)^3];
h_vec_m = [1 1/hus(2) 1/hus(2)^2 1/hus(2)^3];
H_p3 = diag(h_vec_p);
H_m3 = diag(h_vec_m);
X3h_p = data_mat_p(:,2:5)*H_p3;
X3h_m = data_mat_m(:,2:5)*H_m3;
Kvec_h_p = (1-abs(data_mat_p(:,3))/hus(1)).*(abs(data_mat_p(:,3))<hus(1));
Kvec_h_m = (1-abs(data_mat_m(:,3))/hus(2)).*(abs(data_mat_m(:,3))<hus(2));
Kind_h_p = find(Kvec_h_p ~= 0);
Kind_h_m = find(Kvec_h_m ~= 0);
yh_p = yp(Kind_h_p,1);
yh_m = ym(Kind_h_m,1);
X3h_p2 = X3h_p(Kind_h_p,1:2);
X3h_m2 = X3h_m(Kind_h_m,1:2);
Kvec_h_p2 = Kvec_h_p(Kind_h_p,1);
Kvec_h_m2 = Kvec_h_m(Kind_h_m,1);
Wh_p = diag(Kvec_h_p2);
Wh_m = diag(Kvec_h_m2);
Gamma_h_p = X3h_p2'*Wh_p*X3h_p2/n;
Gamma_h_m = X3h_m2'*Wh_m*X3h_m2/n;
invGamma_h_p = invpd(Gamma_h_p);
invGamma_h_m = invpd(Gamma_h_m);
delta_p_opt = e0'*invGamma_h_p*X3h_p2'*Wh_p*yh_p/n;
delta_m_opt = e0'*invGamma_h_m*X3h_m2'*Wh_m*yh_m/n;
tau_us = delta_p_opt-delta_m_opt;
beta_p = invGamma_h_p*X3h_p2'*Wh_p*yh_p/n;
beta_m = invGamma_h_m*X3h_m2'*Wh_m*yh_m/n;
YT_p_hat = X3h_p2*beta_p;
s2_p = (yh_p - YT_p_hat)'*Wh_p*(yh_p - YT_p_hat)/trace(Wh_p-Wh_p*X3h_p2*invpd(X3h_p2'*Wh_p*X3h_p2)*X3h_p2'*Wh_p);

YT_m_hat = X3h_m2*beta_m;
s2_m = (yh_m - YT_m_hat)'*Wh_m*(yh_m - YT_m_hat)/trace(Wh_m-Wh_m*X3h_m2*invpd(X3h_m2'*Wh_m*X3h_m2)*X3h_m2'*Wh_m);

Psi_h_p = s2_p*X3h_p2'*Wh_p*Wh_p*X3h_p2/n;
Psi_h_m = s2_m*X3h_m2'*Wh_m*Wh_m*X3h_m2/n;
Vh_p = (e0'*invGamma_h_p*Psi_h_p*invGamma_h_p*e0)/n;
Vh_m = (e0'*invGamma_h_m*Psi_h_m*invGamma_h_m*e0)/n;
V1 = Vh_p + Vh_m;


% Construct bias-corrected t-value
tval_r = (tau-bias_opt)/se_r;
pval = 2*(1-normcdf(abs(tval_r)));

se_c = sqrt(V1);

output.h = h_mmse;
output.num = num;
output.tau = tau;
output.se_r = se_r;
output.tval_r = tval_r;
output.pval = pval;
output.ci_r = [tau-bias_opt-1.96*se_r tau-bias_opt+1.96*se_r];
output.ci_us = [tau_us-1.96*se_c tau_us+1.96*se_c];

%------------------------------------------------------------------
% End function rddci_mmse_homo_v2
%------------------------------------------------------------------

function fhat = kdensity_est(X,c,h)

% Purpose: estimate density at a point with Epanechnikov kernel
% --------------------------------------------
% Usage: results = density_est
%       X = a column vector of scalar random variables
%       c = point to estimate
%       h = bandwidth
% --------------------------------------------

nobs = size(X,1);
A_h = (X-c)/h;
ExpA_h = 3*sum((abs(A_h)<1).*(1-A_h.^2))/4;
fhat = ExpA_h/nobs/h;

%------------------------------------------------------------------
% End private function density_est
%------------------------------------------------------------------

function f1hat = density1d_est(X,c,h)

% Purpose: estimate 1st derivative density at a point
% --------------------------------------------
% Usage: results = density_est
%       X = a column vector of scalar random variables
%       c = point to estimate
%       h = bandwidth
% --------------------------------------------

nobs = size(X,1);
A_h = (c-X)/h;
B_h = A_h.^2;
k2 =1/5;
const = 3/4;
ExpA_h = (-1)*const*sum(A_h.*(1-B_h).*(abs(A_h)<1))/k2;
f1hat = ExpA_h/nobs/h^2;

%------------------------------------------------------------------
% End private function density1d_est
%------------------------------------------------------------------


